/* Copyright (C) 2006-2015 Michel Gutierrez <mig@downloadhelper.net>
 * All Rights Reserved
 * 
 * If you are interested in methods used in this project, follow the
 * open source project at https://github.com/mi-g/fang
 */


var groups = {}
	
function MakeGroups(selector,type,options) {

	var nodes = document.querySelectorAll(selector);
	for(var i=0; i<nodes.length && (options.maxHits==0 || i<options.maxHits); i++) {
		var node=nodes.item(i);
		var href = null;
		switch(type) {
		case 'link':
			href = node.getAttribute("href").trim();
			break;
		case 'image':
			if(node.width && node.height && type=="image" && Math.min(node.width,node.height)>=options.minImgSize)
				href = node.getAttribute("src").trim();
			break;
		}
		if(!href)
			continue;
		var fileExtension = null
		var m = /\.([^\.]{1,5})$/.exec(href);
		if(m)
			fileExtension = m[1];
		var ancestors = [];
		var node0 = node;
		while(node0 && node0!=document) {
			ancestors.unshift(node0.nodeName.toLowerCase());
			node0 = node0.parentNode;
		}
		var key = HashCode(window.location.href+"@"+ancestors.join("/")+"@"+fileExtension);
		var attr = "vdh-"+key;
		var group = groups[key]; 
		if(!group) {
			group = groups[key] = {
				urlsMap: {},
				urls: [],
				baseUrl: window.location.href,
				extensions: {},
				selectorAttr: attr,
				type: type,
			} 
		}
		if(!group.urlsMap[href]) {
			if(node.firstChild && node.firstChild.nodeName.toLowerCase()=="img")
				node = node.firstChild;
			node.setAttribute(group.selectorAttr,"");
			group.urlsMap[href] = 1;
			group.urls.push(href);
			if(fileExtension) {
				var extension = group.extensions[fileExtension];
				if(extension)
					group.extensions[fileExtension]++;
				else
					group.extensions[fileExtension]=1;
			}
		}
	}
}


self.port.on("detect", function(message) {
	groups = {};
	var selectors = [];
	message.extensions.split("|").forEach(function(extension) {
		selectors.push("a[href$='"+extension+"']");
	});
	MakeGroups(selectors.join(","),"link",message);
	if(message.scanImages)
		MakeGroups("img[src]","image",message);
	
	var groupCount = 0;
	for(var gid in groups) {
		var group = groups[gid];
		if(group.urls.length<message.minFilesPerGroup) {
			delete groups[gid];
			continue;
		}
		groupCount++;
	}
	if(groupCount>0)
		self.port.emit("detected",{
			groups: groups,
		});
});

function HashCode(str) {
	var hash=0, i, chr, len;
	if(str.length==0) 
		return hash;
	for(i=0, len = str.length; i < len; i++) {
		chr = str.charCodeAt(i);
		hash = ((hash << 5) - hash) + chr;
		hash |= 0;
	}
	return ""+Math.abs(hash);
};
