/* Copyright (C) 2006-2015 Michel Gutierrez <mig@downloadhelper.net>
 * All Rights Reserved
 * 
 * If you are interested in methods used in this project, follow the
 * open source project at https://github.com/mi-g/fang
 */


const $chrome = require("chrome");
const Cc = $chrome.Cc;
const Ci = $chrome.Ci;
const Cu = $chrome.Cu;
const self = require("sdk/self");
const Worker = require('sdk/content/worker').Worker;
const windowUtils = require("sdk/window/utils");
const simplePrefs = require('sdk/simple-prefs');
const timers = require("sdk/timers");
const privateBrowsing = require("sdk/private-browsing");
const _ = require("sdk/l10n").get;
const pageMod = require("sdk/page-mod");
const $viewCore = require("sdk/view/core");
const viewFor = $viewCore.viewFor; 

const hits = require("hits");

function CheckGalleries(window) {
	var worker = new Worker({
		window: window,
	    contentScriptFile: self.data.url("gallery-content.js"),
	    onAttach: function(event) {
	    	timers.setTimeout(function() {
		    	worker.port.emit("detect",{
		    		extensions: simplePrefs.prefs['medialink-extensions'],
		    		maxHits: simplePrefs.prefs['medialink-max-hits'],
		    		minFilesPerGroup: simplePrefs.prefs['medialink-min-files-per-group'],
		    		minImgSize: simplePrefs.prefs['medialink-min-img-size'],
		    		scanImages: simplePrefs.prefs['medialink-scan-images'],
		    	});	    		    		
	    	});
	    }
	});
    worker.port.on("detected",function(message) {
    	var hitDataArray = [];
    	
    	Cu.import("resource://gre/modules/NetUtil.jsm");

    	for(var i in message.groups) {
    		var group = message.groups[i];
    		var title = _(group.type=="image"?"gallery":"gallery-links"); 
    		var m = /^https?:\/\/([^\/:]+)/.exec(group.baseUrl);
    		if(m)
				title = _(group.type=="image"?"gallery-from-domain":"gallery-links-from-domain",m[1]);
    		var urls = [];
    		group.urls.forEach(function(url) {
    			urls.push(NetUtil.newURI(group.baseUrl).resolve(url));
    		});
    		var hitData = {
				id: "gallery:"+i,
				title: title,
		    	pageUrl: group.baseUrl,
    			topUrl: group.baseUrl,
    	    	isPrivate: privateBrowsing.isPrivate(worker),
    	    	urls: urls,
    	    	_trackMouse: true,
    	    	_gallery: {
    	    		selectorAttr: group.selectorAttr,
    	    		baseUrl: group.baseUrl,
    	    	}
			}
	    	if(worker.tab) {
				hitData.thumbnail = worker.tab.getThumbnail();
		    	hitData.topUrl = worker.tab.url;
	    	}
    		var extensions = Object.keys(group.extensions);
    		extensions.sort(function(a,b) {
    			return group.extensions[a]-group.extensions[b];
    		});
    		var extStrs = [];
    		extensions.forEach(function(extension) {
    			var str = _('number-type',""+group.extensions[extension],extension.toUpperCase());
    			extStrs.push(str);
    		});
    		hitData.description = _('gallery-files-types',extStrs.join(", "));
    		hitDataArray.push(hitData);
    	}
    	hits.newData(hitDataArray);
    });
}

exports.checkCurrentPage = function() {
	var currentWindow = windowUtils.getMostRecentBrowserWindow();
	if(currentWindow.content)
		CheckGalleries(currentWindow.content);
}

var pageModSelect = null;

const selectedStyle = "border: 5px dashed Red;";

exports.select = function(specs) {

	if(pageModSelect) { 
		pageModSelect.destroy();
		pageModSelect = null;
	}
	
	if(specs.select) {
		pageModSelect = pageMod.PageMod({
			include: specs.gallery.baseUrl,
			attachTo: ["existing","top","frame"],
			contentStyle: [
			    "["+specs.gallery.selectorAttr+"] {"+selectedStyle+"}",
			],
		});
	}
}

var pageModAuto = null;

function StartStop() {
	var autoDetect = simplePrefs.prefs['medialink-auto-detect'];
	
	if(autoDetect && !pageModAuto)
		pageModAuto = pageMod.PageMod({
			include: /^https?.*/,
			attachTo: ["top"],
			contentScriptWhen: "end",
			onAttach: function(worker) {
				if(worker.tab && worker.tab.window)
					CheckGalleries(viewFor(worker.tab.window).content);
			},
		});
	else if(!autoDetect && pageModAuto)
		pageModAuto.destroy();

}
simplePrefs.on("medialink-auto-detect",function() {
	StartStop();
});
StartStop();

